module Natural

import Unitful

using Unitful:@unit

export lₛ, mₛ, tₛ
export lₚ, mₚ, tₚ, Tₚ, qₚ, Qₚ, item

# Additional constants (not in Unitful)
kₑ = 1 / (4 * π * Unitful.ϵ0) # Coulomb constant
gₑ = − 2.00231930436256 # Electron g-factor
gₚ = 5.5856946893 # Proton g-factor
α = Unitful.q^2 / (4 * π * Unitful.ϵ0 * Unitful.ħ * Unitful.c) # fine structure constant
a₀= Unitful.ħ / (Unitful.me * Unitful.c * α) # bohr radius
ΔEₕ = 2 * gₑ * α^4 * gₚ * Unitful.me^2 * Unitful.c^2 / (3 * Unitful.mp) # hydrogen hyperfine transition energy
HHz = -ΔEₕ / Unitful.h # Hydrogen hyperfine transition frequency
bit = Unitful.k * log(2)
hexit = Unitful.k * log(6)
natit = Unitful.k * log(1/α)

# Natural Units
@unit lₛ "lₛ" StoneyLength (sqrt(Unitful.G * kₑ * Unitful.q^2 / Unitful.c^4) |> Unitful.upreferred) false
@unit mₛ "mₛ" StoneyMass (sqrt(kₑ * Unitful.q^2 / Unitful.G) |> Unitful.upreferred) false
@unit tₛ "tₛ" StoneyTime (sqrt(Unitful.G * kₑ * Unitful.q^2 / Unitful.c^6) |> Unitful.upreferred) false

@unit lₚ "lₚ" PlanckLength sqrt(Unitful.ħ * Unitful.G / Unitful.c^3) false
@unit mₚ "mₚ" PlanckMass sqrt(Unitful.ħ * Unitful.c / Unitful.G) false
@unit tₚ "tₚ" PlanckTime sqrt(Unitful.ħ * Unitful.G / Unitful.c^5) false
@unit Tₚ "Tₚ" PlanckTemperature sqrt(Unitful.ħ * Unitful.c ^ 5 / (Unitful.G * Unitful.k ^ 2)) false
@unit qₚ "qₚ" Planckcharge sqrt(4π * Unitful.ϵ0 * Unitful.ħ * Unitful.c) false
@unit Qₚ "Qₚ" PlanckCharge sqrt(Unitful.ϵ0 * Unitful.ħ * Unitful.c) false
item = @unit ç "ç" Count (1 / Unitful.Na) false

end # module Natural
