module Seximal

export @b6_str
export yocte, zepte, atte, femte, pice, nane, micre, mille, cente, dice
export yotte, zette, exe, pete, tere, gige, mege, kile, hecte, dece

# Senary prefixes
function base6(number::AbstractString)
    if occursin('e', number)
        (digits, exponent) = split(number, 'e')
    else
        (digits, exponent) = (number, "0")
    end

    if occursin('.', digits)
        value = parse(BigFloat, digits, base=6)
    else
        value = parse(BigInt, digits, base=6)
    end

    expval = parse(Int, exponent, base=6)

    if expval < 0
        return 1 // (value * BigInt(6) ^ (-expval))
    else
        return value * BigInt(6) ^ expval
    end
end

macro b6_str(digits)
    value = base6(digits)

    return :( $value )
end

quecte = b6"1e-64" # q |
ronte = b6"1e-60" # r  |
yocte = b6"1e-52" # y  |
zepte = b6"1e-44" # z  |
atte = b6"1e-40" # a   |
femte = b6"1e-32" # f  |
pice = b6"1e-24" # p   |
nane = b6"1e-20" # n   |
micre = b6"1e-12" # μ  |
mille = b6"1e-4" # m   |
cente = b6"1e-2" # c   |
dice = b6"1e-1" # d    |
dece = b6"1e1" # de    |
hecte = b6"1e2" # h    |
kile = b6"1e4" # k     |
mege = b6"1e12" # M    |
gige = b6"1e20" # G    |
tere = b6"1e24" # T    |
pete = b6"1e32" # P    |
exe = b6"1e40" # E     |
zette = b6"1e44" # Z   |
yotte = b6"1e52" # Y   |
ronne = b6"1e60" # R   |
quette = b6"1e64" # Q  |

#=
def longdiv(numerator,denominator):
    digits = []
    remainders = [0]
    n = numerator
    while n not in remainders:              #   until repeated remainder or no remainder
        remainders.append(n)                #   add remainder to collection
        digits.append(n//denominator)       #   add integer division to result
        n = n%denominator * 10              #   remainder*10 for next iteration

    #   Result
    result = list(map(str,digits))          #   convert digits to strings
    result = ''.join(result)                #   combine list to string

    if not n:
        result = result[:1]+'.'+result[1:]  #   Insert . into string
    else:
        recurring = remainders.index(n)-1   #   first recurring digit
        #   Insert '.' and then surround recurring part in brackets:
        result = result[:1]+'.'+result[1:recurring]+'['+result[recurring:]+']'

    return result;

print(longdiv(31,8))    #   3.875
print(longdiv(2,13))    #   0.[153846]
print(longdiv(13,14))   #   0.9[285714]

=#

"Converts x to an exact seximal number. Repeated digits are shown in brackets."
function to₆(x :: Rational{BigInt})
    if x < 0
        return "-" * to₆(-x)
    end

    if x > 1
        intpart = floor(x)
        return string(intpart.num, base=6) * to₆(x - intpart)
    end

    digits = []
    remainders = [BigInt(0)]
    remset = Set{BigInt}()
    n = x.num
    d = x.den
    while !(n in remset)
        append!(remainders, n)
        push!(remset, n)
        append!(digits, floor(n // d).num)
        n = (n % d) * 6
    end

    if n == 0
        return "." * prod(map(string, digits[2:end]))
    end

    recur = findfirst(==(n), remainders) - 1
    return "." * prod(map(string, digits[2:recur - 1])) * "(" * prod(map(string, digits[recur:end])) * ")"
end

to₆(x) = to₆(Rational{BigInt}(x))

# TODO: custom base6 version of @prefixed_unit_symbol

end # module Seximal
